"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OUTLINE_TREE_NODE_HEIGHT = exports.OutlineNode = void 0;
const tslib_1 = require("tslib");
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const react_1 = tslib_1.__importDefault(require("react"));
const ide_components_1 = require("@opensumi/ide-components");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const document_symbol_1 = require("@opensumi/ide-editor/lib/browser/breadcrumb/document-symbol");
const outline_node_define_1 = require("./outline-node.define");
const outline_node_module_less_1 = tslib_1.__importDefault(require("./outline-node.module.less"));
const OutlineNode = ({ item, defaultLeftPadding = 8, leftPadding = 8, onClick, onTwistierClick, itemType, decorationService, decorations, }) => {
    const decoration = outline_node_define_1.OutlineTreeNode.is(item) ? decorationService.getDecoration(item) : null;
    const styles_expansion_toggle = (0, ide_core_browser_1.useDesignStyles)(outline_node_module_less_1.default.expansion_toggle, 'expansion_toggle');
    const styles_outline_node = (0, ide_core_browser_1.useDesignStyles)(outline_node_module_less_1.default.outline_node, 'outline_node');
    const handleClick = (ev) => {
        if (itemType === ide_components_1.TreeNodeType.TreeNode || itemType === ide_components_1.TreeNodeType.CompositeTreeNode) {
            onClick(ev, item, itemType);
        }
    };
    const handlerTwistierClick = (ev) => {
        if (itemType === ide_components_1.TreeNodeType.TreeNode || itemType === ide_components_1.TreeNodeType.CompositeTreeNode) {
            if (onTwistierClick) {
                onTwistierClick(ev, item, itemType);
            }
            else {
                onClick(ev, item, itemType);
            }
        }
    };
    const paddingLeft = `${defaultLeftPadding + (item.depth || 0) * (leftPadding || 0) + (!outline_node_define_1.OutlineCompositeTreeNode.is(item) ? 12 : 0)}px`;
    const editorNodeStyle = {
        color: decoration ? decoration.color : '',
        height: exports.OUTLINE_TREE_NODE_HEIGHT,
        lineHeight: `${exports.OUTLINE_TREE_NODE_HEIGHT}px`,
        paddingLeft,
    };
    const renderIcon = (node) => (react_1.default.createElement("div", { className: (0, classnames_1.default)(outline_node_module_less_1.default.icon, node.icon), style: { height: exports.OUTLINE_TREE_NODE_HEIGHT, lineHeight: `${exports.OUTLINE_TREE_NODE_HEIGHT}px` } }));
    const getName = (node) => node.displayName.replace(/\n/g, '↵');
    const renderDisplayName = (node) => (react_1.default.createElement("div", { className: (0, classnames_1.default)(outline_node_module_less_1.default.outline_node_segment, outline_node_module_less_1.default.outline_node_displayname, {
            [outline_node_module_less_1.default.deprecated]: node.raw.tags && node.raw.tags.indexOf(document_symbol_1.SymbolTag.Deprecated) >= 0,
        }) }, getName(node)));
    const renderStatusTail = () => (react_1.default.createElement("div", { className: (0, classnames_1.default)(outline_node_module_less_1.default.outline_node_segment, outline_node_module_less_1.default.outline_node_tail) }, renderBadge()));
    const renderBadge = () => {
        if (!decoration) {
            return null;
        }
        return react_1.default.createElement("div", { className: outline_node_module_less_1.default.outline_node_status }, decoration.badge.slice());
    };
    const renderFolderToggle = (node, clickHandler) => (react_1.default.createElement("div", { onClick: clickHandler, className: (0, classnames_1.default)(outline_node_module_less_1.default.file_tree_node_segment, styles_expansion_toggle, (0, ide_core_browser_1.getIcon)('arrow-right'), {
            [`${outline_node_module_less_1.default.mod_collapsed}`]: !node.expanded,
        }) }));
    const renderTwice = (item) => {
        if (outline_node_define_1.OutlineCompositeTreeNode.is(item)) {
            return renderFolderToggle(item, handlerTwistierClick);
        }
    };
    const getItemTooltip = () => {
        let tooltip = item.tooltip || item.displayName;
        if (decoration && decoration.tooltip) {
            tooltip += ` • ${decoration.tooltip}`;
        }
        return tooltip;
    };
    const renderDescription = (node) => {
        if (outline_node_define_1.OutlineCompositeTreeNode.is(node)) {
            return null;
        }
        return (react_1.default.createElement("div", { className: (0, classnames_1.default)(outline_node_module_less_1.default.outline_node_segment_grow, outline_node_module_less_1.default.outline_node_description) }, node.description));
    };
    return (react_1.default.createElement("div", { key: item.id, onClick: handleClick, title: getItemTooltip(), className: (0, classnames_1.default)(styles_outline_node, decorations ? decorations.classlist : null), style: editorNodeStyle, "data-id": item.id },
        react_1.default.createElement("div", { className: outline_node_module_less_1.default.outline_node_content },
            renderTwice(item),
            renderIcon(item),
            react_1.default.createElement("div", { className: outline_node_module_less_1.default.outline_node_overflow_wrap },
                renderDisplayName(item),
                renderDescription(item)),
            renderStatusTail())));
};
exports.OutlineNode = OutlineNode;
exports.OUTLINE_TREE_NODE_HEIGHT = 22;
//# sourceMappingURL=outline-node.js.map