"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutlineContribution = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const contextkey_1 = require("@opensumi/ide-core-browser/lib/contextkey");
const explorer_contribution_1 = require("@opensumi/ide-explorer/lib/browser/explorer-contribution");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
const common_1 = require("../common");
const outline_1 = require("./outline");
const outline_model_service_1 = require("./services/outline-model.service");
const outline_tree_service_1 = require("./services/outline-tree.service");
let OutlineContribution = class OutlineContribution {
    onDidRender() {
        this.mainLayoutService.collectViewComponent({
            component: outline_1.OutlinePanel,
            collapsed: true,
            id: common_1.OUTLINE_VIEW_ID,
            name: (0, ide_core_browser_1.localize)('outline.title'),
        }, explorer_contribution_1.EXPLORER_CONTAINER_ID);
    }
    registerCommands(registry) {
        registry.registerCommand({
            id: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_COLLAPSE_ALL.id,
            iconClass: (0, ide_core_browser_1.getIcon)('collapse-all'),
            label: (0, ide_core_browser_1.localize)('outline.collapse.all'),
        }, {
            execute: () => {
                this.outlineTreeModelService.collapseAll();
            },
        });
        registry.registerCommand({
            id: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_FOLLOW_CURSOR.id,
            iconClass: (0, ide_core_browser_1.getIcon)('follow-cursor'),
            label: (0, ide_core_browser_1.localize)('outline.follow.cursor'),
        }, {
            execute: () => {
                this.outlineTreeService.followCursor = !this.outlineTreeService.followCursor;
            },
        });
        registry.registerCommand({
            id: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_SORT_KIND.id,
            label: (0, ide_core_browser_1.localize)('outline.sort.kind'),
        }, {
            execute: () => {
                this.outlineTreeService.sortType = 2 /* OutlineSortOrder.ByKind */;
            },
        });
        registry.registerCommand({
            id: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_SORT_NAME.id,
            label: (0, ide_core_browser_1.localize)('outline.sort.name'),
        }, {
            execute: () => {
                this.outlineTreeService.sortType = 1 /* OutlineSortOrder.ByName */;
            },
        });
        registry.registerCommand({
            id: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_SORT_POSITION.id,
            label: (0, ide_core_browser_1.localize)('outline.sort.position'),
        }, {
            execute: () => {
                this.outlineTreeService.sortType = 0 /* OutlineSortOrder.ByPosition */;
            },
        });
    }
    registerToolbarItems(registry) {
        registry.registerItem({
            id: 'outline.action.follow.cursor',
            viewId: 'outline-view',
            command: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_FOLLOW_CURSOR.id,
            label: (0, ide_core_browser_1.localize)('outline.follow.cursor'),
            toggledWhen: contextkey_1.OutlineFollowCursorContext.raw,
        });
        registry.registerItem({
            id: 'outline.action.collapse.all',
            viewId: 'outline-view',
            command: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_COLLAPSE_ALL.id,
            label: (0, ide_core_browser_1.localize)('outline.collapse.all'),
        });
        registry.registerItem({
            id: 'outline.menu.sort.kind',
            viewId: 'outline-view',
            command: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_SORT_KIND.id,
            group: 'inline',
            toggledWhen: `${contextkey_1.OutlineSortTypeContext.raw} == ${2 /* OutlineSortOrder.ByKind */}`,
        });
        registry.registerItem({
            id: 'outline.menu.sort.name',
            viewId: 'outline-view',
            command: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_SORT_NAME.id,
            group: 'inline',
            toggledWhen: `${contextkey_1.OutlineSortTypeContext.raw} == ${1 /* OutlineSortOrder.ByName */}`,
        });
        registry.registerItem({
            id: 'outline.menu.sort.position',
            viewId: 'outline-view',
            command: ide_core_browser_1.OUTLINE_COMMANDS.OUTLINE_SORT_POSITION.id,
            group: 'inline',
            toggledWhen: `${contextkey_1.OutlineSortTypeContext.raw} == ${0 /* OutlineSortOrder.ByPosition */}`,
        });
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_main_layout_1.IMainLayoutService),
    tslib_1.__metadata("design:type", Object)
], OutlineContribution.prototype, "mainLayoutService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", outline_tree_service_1.OutlineTreeService)
], OutlineContribution.prototype, "outlineTreeService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", outline_model_service_1.OutlineModelService)
], OutlineContribution.prototype, "outlineTreeModelService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IContextKeyService),
    tslib_1.__metadata("design:type", Object)
], OutlineContribution.prototype, "contextKey", void 0);
OutlineContribution = tslib_1.__decorate([
    (0, ide_core_browser_1.Domain)(ide_main_layout_1.MainLayoutContribution, ide_core_browser_1.TabBarToolbarContribution, ide_core_browser_1.CommandContribution)
], OutlineContribution);
exports.OutlineContribution = OutlineContribution;
//# sourceMappingURL=outline.contribution.js.map