"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutlineTreeView = exports.OutlinePanel = void 0;
const tslib_1 = require("tslib");
const debounce_1 = tslib_1.__importDefault(require("lodash/debounce"));
const react_1 = tslib_1.__importStar(require("react"));
const ide_components_1 = require("@opensumi/ide-components");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const progress_bar_1 = require("@opensumi/ide-core-browser/lib/progress/progress-bar");
const react_hooks_1 = require("@opensumi/ide-core-browser/lib/react-hooks");
const outline_node_1 = require("./outline-node");
const outline_module_less_1 = tslib_1.__importDefault(require("./outline.module.less"));
const outline_model_service_1 = require("./services/outline-model.service");
const OutlinePanel = ({ viewState }) => {
    const { height } = viewState;
    const wrapperRef = (0, react_1.useRef)(null);
    const outlineModelService = (0, react_hooks_1.useInjectable)(outline_model_service_1.OutlineModelService);
    const [model, setModel] = (0, react_1.useState)(undefined);
    const handleTreeReady = (0, react_1.useCallback)((handle) => {
        outlineModelService.handleTreeHandler(Object.assign(Object.assign({}, handle), { hasDirectFocus: () => wrapperRef.current === document.activeElement }));
    }, [outlineModelService, wrapperRef.current]);
    const handleItemClicked = (0, react_1.useCallback)((ev, item, type) => {
        // 阻止点击事件冒泡
        ev.stopPropagation();
        const { handleItemClick } = outlineModelService;
        if (!item) {
            return;
        }
        handleItemClick(item, type);
    }, [outlineModelService]);
    const handleTwistierClicked = (0, react_1.useCallback)((ev, item) => {
        // 阻止点击事件冒泡
        ev.stopPropagation();
        const { toggleDirectory } = outlineModelService;
        if (!item) {
            return;
        }
        toggleDirectory(item);
    }, [outlineModelService]);
    const handleOuterClick = (0, react_1.useCallback)(() => {
        // 空白区域点击，取消焦点状态
        const { enactiveNodeDecoration } = outlineModelService;
        enactiveNodeDecoration();
    }, [outlineModelService]);
    (0, react_1.useEffect)(() => {
        setModel(outlineModelService.treeModel);
        const disposable = outlineModelService.onDidUpdateTreeModel((model) => {
            setModel(model);
        });
        return () => {
            disposable.dispose();
        };
    }, []);
    (0, react_1.useEffect)(() => {
        var _a;
        const handleBlur = () => {
            outlineModelService.handleTreeBlur();
        };
        (_a = wrapperRef.current) === null || _a === void 0 ? void 0 : _a.addEventListener('blur', handleBlur, true);
        return () => {
            var _a;
            (_a = wrapperRef.current) === null || _a === void 0 ? void 0 : _a.removeEventListener('blur', handleBlur, true);
            outlineModelService.handleTreeBlur();
        };
    }, [wrapperRef.current]);
    const [loadingState, setLoadingState] = react_1.default.useState(false);
    const loading = react_1.default.useMemo(() => loadingState, [loadingState]);
    (0, react_1.useEffect)(() => {
        const disposable1 = outlineModelService.onLoadingStateChange((0, debounce_1.default)((current) => {
            setLoadingState((previous) => (previous !== current ? current : previous));
        }, 16 * 5, {
            leading: true,
        }));
        return () => {
            disposable1.dispose();
        };
    }, [setLoadingState]);
    return (react_1.default.createElement("div", { className: outline_module_less_1.default.outline_container, tabIndex: -1, ref: wrapperRef, onClick: handleOuterClick },
        react_1.default.createElement(progress_bar_1.Progress, { loading: loading }),
        react_1.default.createElement(exports.OutlineTreeView, { loading: loading, height: height, model: model, onItemClick: handleItemClicked, onTwistierClick: handleTwistierClicked, onDidTreeReady: handleTreeReady })));
};
exports.OutlinePanel = OutlinePanel;
exports.OutlineTreeView = (0, react_1.memo)(({ height, model, onItemClick, onTwistierClick, onDidTreeReady, loading }) => {
    const outlineModelService = (0, react_hooks_1.useInjectable)(outline_model_service_1.OutlineModelService);
    const { decorationService, commandService } = outlineModelService;
    const renderTreeNode = (0, react_1.useCallback)((props) => (react_1.default.createElement(outline_node_1.OutlineNode, { item: props.item, itemType: props.itemType, decorationService: decorationService, commandService: commandService, decorations: outlineModelService.decorations.getDecorations(props.item), onClick: onItemClick, onTwistierClick: onTwistierClick, defaultLeftPadding: 8, leftPadding: 8 })), [model]);
    if (!model) {
        return (react_1.default.createElement("span", { className: outline_module_less_1.default.outline_empty_text }, loading ? (0, ide_core_browser_1.localize)('common.loading') : (0, ide_core_browser_1.localize)('outline.nomodel')));
    }
    else {
        return (react_1.default.createElement(ide_components_1.RecycleTree, { height: height, itemHeight: outline_node_1.OUTLINE_TREE_NODE_HEIGHT, onReady: onDidTreeReady, model: model, placeholder: () => react_1.default.createElement("span", { className: outline_module_less_1.default.outline_empty_text }, (0, ide_core_browser_1.localize)('outline.noinfo')) }, renderTreeNode));
    }
});
exports.OutlineTreeView.displayName = 'OutlineTreeView';
//# sourceMappingURL=outline.js.map