"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutlineDecorationService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_theme_1 = require("@opensumi/ide-theme");
let OutlineDecorationService = class OutlineDecorationService {
    constructor() {
        this._diagnosisInfo = [];
    }
    updateDiagnosisInfo(uri) {
        this._diagnosisInfo = uri ? this.markerManager.getMarkers({ resource: uri.toString(), opened: true }) : [];
    }
    getDecoration(node) {
        const markers = [];
        let topMarker;
        // 根据 node.raw.range 判断相交情况
        // 这里的相交判断实际上可以做一下数据裁剪
        const diagnosisInfos = this._diagnosisInfo.filter((marker) => {
            if (marker.startLineNumber <= node.raw.range.startLineNumber &&
                marker.endLineNumber >= node.raw.range.startLineNumber) {
                return true;
            }
            else if (node.raw.range.startLineNumber <= marker.startLineNumber &&
                node.raw.range.endLineNumber >= marker.startLineNumber) {
                return true;
            }
            return false;
        });
        for (const marker of diagnosisInfos) {
            if (marker.severity === ide_core_browser_1.MarkerSeverity.Error || marker.severity === ide_core_browser_1.MarkerSeverity.Warning) {
                markers.push(marker);
            }
            if (!topMarker || marker.severity > topMarker.severity) {
                topMarker = marker;
            }
        }
        return {
            color: (topMarker === null || topMarker === void 0 ? void 0 : topMarker.severity) === ide_core_browser_1.MarkerSeverity.Error || (topMarker === null || topMarker === void 0 ? void 0 : topMarker.severity) === ide_core_browser_1.MarkerSeverity.Warning
                ? this.themeService.getColor({
                    id: topMarker.severity === ide_core_browser_1.MarkerSeverity.Error ? ide_theme_1.listErrorForeground : ide_theme_1.listWarningForeground,
                })
                : undefined,
            tooltip: topMarker === null || topMarker === void 0 ? void 0 : topMarker.message,
            badge: markers.length > 0 ? (markers.length > 9 ? '9+' : markers.length + '') : '',
        };
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", ide_core_browser_1.MarkerManager)
], OutlineDecorationService.prototype, "markerManager", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_theme_1.IThemeService),
    tslib_1.__metadata("design:type", Object)
], OutlineDecorationService.prototype, "themeService", void 0);
OutlineDecorationService = tslib_1.__decorate([
    (0, di_1.Injectable)()
], OutlineDecorationService);
exports.OutlineDecorationService = OutlineDecorationService;
//# sourceMappingURL=outline-decoration.service.js.map