"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutlineEventService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const browser_1 = require("@opensumi/ide-editor/lib/browser");
const document_symbol_1 = require("@opensumi/ide-editor/lib/browser/breadcrumb/document-symbol");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
const index_1 = require("../../common/index");
let OutlineEventService = class OutlineEventService extends ide_core_browser_1.WithEventBus {
    constructor() {
        super(...arguments);
        this._onDidDocumentSymbolChange = new ide_core_browser_1.Emitter();
        this._onDidSelectionChange = new ide_core_browser_1.Emitter();
        this._onDidEditorActiveResourceStateChange = new ide_core_browser_1.Emitter();
        this._onDidViewCollapseChange = new ide_core_browser_1.Emitter();
    }
    /**
     * on did document symbol changed
     */
    get onDidChange() {
        return this._onDidDocumentSymbolChange.event;
    }
    /**
     * on did editor active resource state changed
     */
    get onDidActiveChange() {
        return this._onDidEditorActiveResourceStateChange.event;
    }
    get onDidSelectionChange() {
        return this._onDidSelectionChange.event;
    }
    get onDidViewCollapseChange() {
        return this._onDidViewCollapseChange.event;
    }
    onEditorActiveResourceStateChangedEvent(e) {
        this._onDidEditorActiveResourceStateChange.fire(e.payload.editorUri);
    }
    onEditorSelectionChangeEvent(e) {
        this._onDidSelectionChange.fire(e.payload.editorUri);
    }
    onDocumentSymbolChange(e) {
        this._onDidDocumentSymbolChange.fire(e.payload);
    }
    onViewCollapseChangedEvent(e) {
        if (e.payload.viewId === index_1.OUTLINE_VIEW_ID) {
            this._onDidViewCollapseChange.fire(e.payload.collapsed);
        }
    }
};
tslib_1.__decorate([
    (0, ide_core_browser_1.OnEvent)(browser_1.EditorActiveResourceStateChangedEvent),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [browser_1.EditorActiveResourceStateChangedEvent]),
    tslib_1.__metadata("design:returntype", void 0)
], OutlineEventService.prototype, "onEditorActiveResourceStateChangedEvent", null);
tslib_1.__decorate([
    (0, ide_core_browser_1.OnEvent)(browser_1.EditorSelectionChangeEvent),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [browser_1.EditorSelectionChangeEvent]),
    tslib_1.__metadata("design:returntype", void 0)
], OutlineEventService.prototype, "onEditorSelectionChangeEvent", null);
tslib_1.__decorate([
    (0, ide_core_browser_1.OnEvent)(document_symbol_1.DocumentSymbolChangedEvent),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [document_symbol_1.DocumentSymbolChangedEvent]),
    tslib_1.__metadata("design:returntype", void 0)
], OutlineEventService.prototype, "onDocumentSymbolChange", null);
tslib_1.__decorate([
    (0, ide_core_browser_1.OnEvent)(ide_main_layout_1.ViewCollapseChangedEvent),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [ide_main_layout_1.ViewCollapseChangedEvent]),
    tslib_1.__metadata("design:returntype", void 0)
], OutlineEventService.prototype, "onViewCollapseChangedEvent", null);
OutlineEventService = tslib_1.__decorate([
    (0, di_1.Injectable)()
], OutlineEventService);
exports.OutlineEventService = OutlineEventService;
//# sourceMappingURL=outline-event.service.js.map