import { DecorationsManager, IRecycleTreeHandle, TreeNodeType } from '@opensumi/ide-components';
import { CommandService, Event, URI } from '@opensumi/ide-core-browser';
import { INormalizedDocumentSymbol } from '@opensumi/ide-editor/lib/browser/breadcrumb/document-symbol';
import { IOutlineDecorationService } from '../../common';
import { OutlineCompositeTreeNode, OutlineTreeNode } from '../outline-node.define';
import { OutlineEventService } from './outline-event.service';
import { OutlineTreeModel } from './outline-model';
export interface IEditorTreeHandle extends IRecycleTreeHandle {
    hasDirectFocus: () => boolean;
}
export declare class OutlineModelService {
    private static DEFAULT_REFRESH_DELAY;
    private static DEFAULT_INIT_TREE_MODEL_DELAY;
    private readonly injector;
    private readonly logger;
    private readonly outlineTreeService;
    private markerManager;
    decorationService: IOutlineDecorationService;
    readonly outlineEventService: OutlineEventService;
    private readonly editorService;
    readonly commandService: CommandService;
    private documentSymbolStore;
    private readonly mainLayoutService;
    private _activeTreeModel;
    private _allTreeModels;
    private _whenInitTreeModelReady;
    private _whenReady;
    private _decorations;
    private _outlineTreeHandle;
    private refreshDeferred;
    private _changeEventDispatchQueue;
    private selectedDecoration;
    private focusedDecoration;
    private _focusedNode;
    private _selectedNodes;
    private disposableCollection;
    private onDidRefreshedEmitter;
    private onLoadingStateChangedEmitter;
    private onDidUpdateTreeModelEmitter;
    private _ignoreFollowCursorUpdateEventTimer;
    private initTreeModelDelayer;
    private refreshDelayer;
    constructor();
    get whenReady(): Promise<void>;
    get flushEventQueuePromise(): Promise<void> | null;
    get outlineTreeHandle(): IEditorTreeHandle;
    get decorations(): DecorationsManager;
    get treeModel(): OutlineTreeModel | undefined;
    get focusedNode(): OutlineCompositeTreeNode | OutlineTreeNode | undefined;
    get selectedNodes(): (OutlineCompositeTreeNode | OutlineTreeNode)[];
    get onLoadingStateChange(): Event<boolean>;
    get onDidRefreshed(): Event<void>;
    get onDidUpdateTreeModel(): Event<OutlineTreeModel | undefined>;
    get whenInitTreeModelReady(): Promise<void>;
    get whenRefreshReady(): Promise<void> | undefined;
    private setTreeModel;
    private doInitTreeModelByCurrentUri;
    initTreeModelByCurrentUri(uri?: URI | null): Promise<void>;
    initTreeModel(): Promise<void>;
    initDecorations(root: any): DecorationsManager;
    private locateSelection;
    private findCurrentDocumentSymbol;
    private positionInRange;
    clearNodeSelectedDecoration: () => void;
    activeNodeDecoration: (target: OutlineCompositeTreeNode | OutlineTreeNode, dispatch?: boolean) => void;
    selectNodeDecoration: (target: OutlineCompositeTreeNode | OutlineTreeNode, dispatch?: boolean) => void;
    activeNodeFocusedDecoration: (target: OutlineCompositeTreeNode | OutlineTreeNode, removePreFocusedDecoration?: boolean) => void;
    activeNodeSelectedDecoration: (target: OutlineCompositeTreeNode | OutlineTreeNode) => void;
    enactiveNodeDecoration: () => void;
    removeNodeDecoration(): void;
    handleTreeHandler(handle: IEditorTreeHandle): void;
    handleTreeBlur: () => void;
    handleItemClick: (item: OutlineCompositeTreeNode | OutlineTreeNode, type: TreeNodeType) => void;
    toggleDirectory: (item: OutlineCompositeTreeNode) => Promise<void>;
    protected revealRange(symbol: INormalizedDocumentSymbol): void;
    /**
     * 刷新指定下的所有子节点
     */
    refresh(): Promise<void>;
    flushEventQueue: () => Promise<null[]> | undefined;
    location: (node: OutlineTreeNode) => Promise<void>;
    collapseAll: () => Promise<void>;
    dispose(): void;
}
//# sourceMappingURL=outline-model.service.d.ts.map