"use strict";
var OutlineModelService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutlineModelService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_components_1 = require("@opensumi/ide-components");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const browser_1 = require("@opensumi/ide-editor/lib/browser");
const document_symbol_1 = require("@opensumi/ide-editor/lib/browser/breadcrumb/document-symbol");
const explorer_contribution_1 = require("@opensumi/ide-explorer/lib/browser/explorer-contribution");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const common_1 = require("../../common");
const outline_node_module_less_1 = tslib_1.__importDefault(require("../outline-node.module.less"));
const outline_event_service_1 = require("./outline-event.service");
const outline_model_1 = require("./outline-model");
const outline_tree_service_1 = require("./outline-tree.service");
const { Path } = ide_core_browser_1.path;
let OutlineModelService = OutlineModelService_1 = class OutlineModelService {
    constructor() {
        this._allTreeModels = new ide_core_browser_1.LRUMap(100, 80);
        this._changeEventDispatchQueue = [];
        // 装饰器
        this.selectedDecoration = new ide_components_1.Decoration(outline_node_module_less_1.default.mod_selected); // 选中态
        this.focusedDecoration = new ide_components_1.Decoration(outline_node_module_less_1.default.mod_focused); // 焦点态
        // 选中态的节点
        this._selectedNodes = [];
        this.disposableCollection = new ide_core_browser_1.DisposableCollection();
        this.onDidRefreshedEmitter = new ide_core_browser_1.Emitter();
        this.onLoadingStateChangedEmitter = new ide_core_browser_1.Emitter();
        this.onDidUpdateTreeModelEmitter = new ide_core_browser_1.Emitter();
        this._ignoreFollowCursorUpdateEventTimer = 0;
        // 清空所有节点选中态
        this.clearNodeSelectedDecoration = () => {
            this._selectedNodes.forEach((file) => {
                this.selectedDecoration.removeTarget(file);
            });
            this._selectedNodes = [];
        };
        // 清空其他选中/焦点态节点，更新当前焦点节点
        this.activeNodeDecoration = (target, dispatch = true) => {
            var _a;
            if (target) {
                for (const target of this.selectedDecoration.appliedTargets.keys()) {
                    this.selectedDecoration.removeTarget(target);
                }
                for (const target of this.focusedDecoration.appliedTargets.keys()) {
                    this.focusedDecoration.removeTarget(target);
                }
                this.selectedDecoration.addTarget(target);
                this.focusedDecoration.addTarget(target);
                this._focusedNode = target;
                this._selectedNodes = [target];
                // 通知视图更新
                dispatch && ((_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.dispatchChange());
            }
        };
        // 清空其他选中/焦点态节点，更新当前选中节点
        this.selectNodeDecoration = (target, dispatch = true) => {
            var _a;
            if (target) {
                if (this.selectedNodes.length > 0) {
                    this.selectedNodes.forEach((node) => {
                        this.selectedDecoration.removeTarget(node);
                    });
                }
                if (this.focusedNode) {
                    this.focusedDecoration.removeTarget(this.focusedNode);
                }
                this.selectedDecoration.addTarget(target);
                this._selectedNodes = [target];
                // 通知视图更新
                dispatch && ((_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.dispatchChange());
            }
        };
        // 清空其他焦点态节点，更新当前焦点节点，
        // removePreFocusedDecoration 表示更新焦点节点时如果此前已存在焦点节点，之前的节点装饰器将会被移除
        this.activeNodeFocusedDecoration = (target, removePreFocusedDecoration = false) => {
            var _a;
            if (this.focusedNode !== target) {
                if (removePreFocusedDecoration) {
                    // 当存在上一次右键菜单激活的文件时，需要把焦点态的文件节点的装饰器全部移除
                    if (this.focusedNode) {
                        // 多选情况下第一次切换焦点文件
                        this.focusedDecoration.removeTarget(this.focusedNode);
                    }
                }
                else if (this.focusedNode) {
                    this.focusedDecoration.removeTarget(this.focusedNode);
                }
                if (target) {
                    this.selectedDecoration.addTarget(target);
                    this.focusedDecoration.addTarget(target);
                    this._focusedNode = target;
                    this._selectedNodes.push(target);
                }
            }
            // 通知视图更新
            (_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.dispatchChange();
        };
        // 选中当前指定节点，添加装饰器属性
        this.activeNodeSelectedDecoration = (target) => {
            var _a;
            if (this._selectedNodes.indexOf(target) > -1) {
                return;
            }
            if (this.selectedNodes.length > 0) {
                this.selectedNodes.forEach((file) => {
                    this.selectedDecoration.removeTarget(file);
                });
            }
            this._selectedNodes = [target];
            this.selectedDecoration.addTarget(target);
            // 通知视图更新
            (_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.dispatchChange();
        };
        // 取消选中节点焦点
        this.enactiveNodeDecoration = () => {
            var _a;
            if (this.focusedNode) {
                this.focusedDecoration.removeTarget(this.focusedNode);
                (_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.dispatchChange();
            }
            this._focusedNode = undefined;
        };
        this.handleTreeBlur = () => {
            // 清空焦点状态
            this.enactiveNodeDecoration();
        };
        this.handleItemClick = (item, type) => {
            // 单选操作默认先更新选中状态
            this.activeNodeDecoration(item);
            this.revealRange(item.raw);
            this._ignoreFollowCursorUpdateEventTimer++;
        };
        this.toggleDirectory = async (item) => {
            if (item.expanded) {
                this.outlineTreeHandle.collapseNode(item);
            }
            else {
                this.outlineTreeHandle.expandNode(item);
            }
        };
        this.flushEventQueue = () => {
            if (!this._changeEventDispatchQueue || this._changeEventDispatchQueue.length === 0) {
                return;
            }
            this._changeEventDispatchQueue.sort((pathA, pathB) => {
                const pathADepth = Path.pathDepth(pathA);
                const pathBDepth = Path.pathDepth(pathB);
                return pathADepth - pathBDepth;
            });
            const roots = [this._changeEventDispatchQueue[0]];
            for (const path of this._changeEventDispatchQueue) {
                if (roots.some((root) => path.indexOf(root) === 0)) {
                    continue;
                }
                else {
                    roots.push(path);
                }
            }
            const promise = (0, ide_core_browser_1.pSeries)(roots.map((path) => async () => {
                var _a, _b;
                const watcher = (_b = (_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.root) === null || _b === void 0 ? void 0 : _b.watchEvents.get(path);
                if (watcher && typeof watcher.callback === 'function') {
                    await watcher.callback({ type: ide_components_1.WatchEvent.Changed, path });
                }
                return null;
            }));
            // 重置更新队列
            this._changeEventDispatchQueue = [];
            return promise;
        };
        this.location = async (node) => {
            var _a;
            await ((_a = this.refreshDeferred) === null || _a === void 0 ? void 0 : _a.promise);
            if (!node) {
                return;
            }
            if (!this.outlineTreeHandle) {
                return;
            }
            node = (await this.outlineTreeHandle.ensureVisible(node, 'smart'));
        };
        this.collapseAll = async () => {
            var _a, _b;
            await ((_a = this.refreshDeferred) === null || _a === void 0 ? void 0 : _a.promise);
            await ((_b = this.treeModel) === null || _b === void 0 ? void 0 : _b.root.collapsedAll());
        };
        this.initTreeModelDelayer = new ide_core_browser_1.ThrottledDelayer(OutlineModelService_1.DEFAULT_INIT_TREE_MODEL_DELAY);
        this.refreshDelayer = new ide_core_browser_1.ThrottledDelayer(OutlineModelService_1.DEFAULT_REFRESH_DELAY);
        this._whenReady = this.initTreeModel();
    }
    get whenReady() {
        return this._whenReady;
    }
    get flushEventQueuePromise() {
        return this.refreshDeferred && this.refreshDeferred.promise;
    }
    get outlineTreeHandle() {
        return this._outlineTreeHandle;
    }
    get decorations() {
        return this._decorations;
    }
    get treeModel() {
        return this._activeTreeModel;
    }
    // 既是选中态，也是焦点态节点
    get focusedNode() {
        return this._focusedNode;
    }
    // 是选中态，非焦点态节点
    get selectedNodes() {
        return this._selectedNodes;
    }
    get onLoadingStateChange() {
        return this.onLoadingStateChangedEmitter.event;
    }
    get onDidRefreshed() {
        return this.onDidRefreshedEmitter.event;
    }
    get onDidUpdateTreeModel() {
        return this.onDidUpdateTreeModelEmitter.event;
    }
    get whenInitTreeModelReady() {
        return this._whenInitTreeModelReady;
    }
    get whenRefreshReady() {
        var _a;
        return (_a = this.refreshDeferred) === null || _a === void 0 ? void 0 : _a.promise;
    }
    setTreeModel(treeModel) {
        this._activeTreeModel = treeModel;
        this.onDidUpdateTreeModelEmitter.fire(this._activeTreeModel);
    }
    async doInitTreeModelByCurrentUri(uri) {
        let treeModel;
        if (uri) {
            // FIXME: 根据是否为多工作区创建不同根节点
            const root = (await this.outlineTreeService.resolveChildren())[0];
            if (!root) {
                this.logger.error(`init outline tree model error: cannot resolve ${uri} children`);
                return;
            }
            treeModel = this.injector.get(outline_model_1.OutlineTreeModel, [root]);
            await treeModel.ensureReady;
            // 初始化节点装饰器
            const decoration = this.initDecorations(root);
            if (this._allTreeModels.has(uri.toString())) {
                this._allTreeModels.delete(uri.toString());
            }
            this._allTreeModels.set(uri.toString(), {
                treeModel,
                decoration,
            });
        }
        this.setTreeModel(treeModel);
        return treeModel;
    }
    async initTreeModelByCurrentUri(uri) {
        var _a;
        await this.outlineTreeService.whenReady;
        // 等待上一次刷新完成
        await ((_a = this.refreshDeferred) === null || _a === void 0 ? void 0 : _a.promise);
        this.outlineTreeService.currentUri = uri;
        if (uri && this._allTreeModels.has(uri.toString())) {
            const treeModelStore = this._allTreeModels.get(uri.toString());
            this._decorations = treeModelStore.decoration;
            this.setTreeModel(treeModelStore.treeModel);
        }
        else {
            await this.doInitTreeModelByCurrentUri(uri);
        }
    }
    async initTreeModel() {
        var _a;
        this._whenInitTreeModelReady = this.initTreeModelByCurrentUri((_a = this.editorService.currentEditor) === null || _a === void 0 ? void 0 : _a.currentUri);
        await this._whenInitTreeModelReady;
        this.disposableCollection.push(this.markerManager.onMarkerChanged((resources) => {
            if (this.outlineTreeService.currentUri &&
                resources.find((resource) => resource === this.outlineTreeService.currentUri.toString())) {
                this.refresh();
            }
        }));
        this.disposableCollection.push(this.outlineEventService.onDidActiveChange(async () => {
            if (!this.initTreeModelDelayer.isTriggered()) {
                this.initTreeModelDelayer.cancel();
            }
            this.onLoadingStateChangedEmitter.fire(true);
            this.initTreeModelDelayer.trigger(async () => {
                var _a;
                await this._whenInitTreeModelReady;
                // 初始化时如果存在还未刷新的执行逻辑时，需要进行清理
                if (!this.refreshDelayer.isTriggered()) {
                    this.refreshDelayer.cancel();
                }
                const uri = (_a = this.editorService.currentEditor) === null || _a === void 0 ? void 0 : _a.currentUri;
                this._whenInitTreeModelReady = this.initTreeModelByCurrentUri(uri);
                this.onLoadingStateChangedEmitter.fire(false);
            });
        }));
        this.disposableCollection.push(this.outlineEventService.onDidSelectionChange(() => {
            // 选取更改时不需要刷新Tree，仅需要定位选择位置即可
            if (this.outlineTreeService.followCursor) {
                // 如果设置了跟随光标，此时查询一下当前焦点节点
                this.locateSelection(false);
            }
        }));
        this.disposableCollection.push(this.outlineEventService.onDidChange((uri) => {
            var _a, _b;
            if (uri === ((_a = this.editorService.currentEditor) === null || _a === void 0 ? void 0 : _a.currentUri)) {
                // 因为刷新 children 那个函数不能传 currentUri，这里利用一下副作用
                this.outlineTreeService.currentUri = (_b = this.editorService.currentEditor) === null || _b === void 0 ? void 0 : _b.currentUri;
                this.refresh();
            }
        }));
        this.disposableCollection.push(this.outlineEventService.onDidViewCollapseChange((collapsed) => {
            if (!collapsed) {
                this.refresh();
            }
        }));
        this.disposableCollection.push(this.outlineTreeService.onDidChange(() => {
            this.refresh();
        }));
        this.disposableCollection.push(ide_core_browser_1.Disposable.create(() => {
            this._allTreeModels.clear();
        }));
    }
    initDecorations(root) {
        this._decorations = new ide_components_1.DecorationsManager(root);
        this._decorations.addDecoration(this.selectedDecoration);
        this._decorations.addDecoration(this.focusedDecoration);
        return this._decorations;
    }
    locateSelection(quiet = true) {
        if (!this.outlineTreeService.currentUri) {
            return;
        }
        const symbols = this.documentSymbolStore.getDocumentSymbol(this.outlineTreeService.currentUri);
        if (symbols) {
            const activeSymbols = this.findCurrentDocumentSymbol(symbols, this.editorService.currentEditorGroup.codeEditor.monacoEditor.getPosition());
            const node = this.outlineTreeService.getTreeNodeBySymbol(activeSymbols[activeSymbols.length - 1]);
            if (node) {
                if (this._ignoreFollowCursorUpdateEventTimer) {
                    this._ignoreFollowCursorUpdateEventTimer--;
                    return;
                }
                else {
                    this.location(node);
                }
                this.activeNodeDecoration(node, !quiet);
            }
        }
    }
    findCurrentDocumentSymbol(documentSymbols, position) {
        const result = [];
        if (!position) {
            return result;
        }
        let toFindIn = documentSymbols;
        while (toFindIn && toFindIn.length > 0) {
            let found = false;
            for (const documentSymbol of toFindIn) {
                if (this.positionInRange(position, documentSymbol.range)) {
                    result.push(documentSymbol);
                    toFindIn = documentSymbol.children;
                    found = true;
                    break;
                }
            }
            if (!found) {
                break;
            }
        }
        return result;
    }
    positionInRange(pos, range) {
        if (pos.lineNumber < range.startLineNumber) {
            return false;
        }
        else if (pos.lineNumber === range.startLineNumber) {
            return pos.column >= range.startColumn;
        }
        else if (pos.lineNumber < range.endLineNumber) {
            return true;
        }
        else if (pos.lineNumber === range.endLineNumber) {
            return pos.column <= range.endColumn;
        }
        else {
            return false;
        }
    }
    removeNodeDecoration() {
        if (!this.decorations) {
            return;
        }
        this.decorations.removeDecoration(this.selectedDecoration);
        this.decorations.removeDecoration(this.focusedDecoration);
    }
    handleTreeHandler(handle) {
        this._outlineTreeHandle = handle;
    }
    revealRange(symbol) {
        const currentEditor = this.editorService.currentEditorGroup.codeEditor;
        currentEditor.monacoEditor.revealLineInCenter(symbol.range.startLineNumber);
        currentEditor.monacoEditor.setPosition(new monaco.Position(symbol.range.startLineNumber, symbol.range.endLineNumber));
    }
    /**
     * 刷新指定下的所有子节点
     */
    async refresh() {
        var _a, _b, _c, _d, _e;
        this.onLoadingStateChangedEmitter.fire(true);
        await this.whenInitTreeModelReady;
        await this.whenRefreshReady;
        const node = (_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.root;
        if (!node || !((_c = (_b = this.editorService) === null || _b === void 0 ? void 0 : _b.currentEditor) === null || _c === void 0 ? void 0 : _c.currentUri)) {
            // 1. 初次加载时可能还没有初始化Tree，主要原因在于没办法在outline加载时准确把握拿到currentEditor的时机
            // 2. 当前没有激活的URI时，需要情况当前的Tree
            this._whenInitTreeModelReady = this.initTreeModelByCurrentUri((_e = (_d = this.editorService) === null || _d === void 0 ? void 0 : _d.currentEditor) === null || _e === void 0 ? void 0 : _e.currentUri);
            this.onLoadingStateChangedEmitter.fire(false);
            return;
        }
        if (!this.refreshDelayer.isTriggered()) {
            this.refreshDelayer.cancel();
        }
        return this.refreshDelayer.trigger(async () => {
            var _a, _b, _c;
            const handler = this.mainLayoutService.getTabbarHandler(explorer_contribution_1.EXPLORER_CONTAINER_ID);
            if (!handler || !handler.isVisible || handler.isCollapsed(common_1.OUTLINE_VIEW_ID)) {
                if (this.refreshDeferred) {
                    this.refreshDeferred.resolve();
                    this.refreshDeferred = null;
                }
                this.onLoadingStateChangedEmitter.fire(false);
                return;
            }
            if (!this.refreshDeferred) {
                this.refreshDeferred = new ide_core_browser_1.Deferred();
            }
            this.outlineTreeService.currentUri = (_b = (_a = this.editorService) === null || _a === void 0 ? void 0 : _a.currentEditor) === null || _b === void 0 ? void 0 : _b.currentUri;
            if (node.currentUri &&
                this.outlineTreeService.currentUri &&
                this.outlineTreeService.currentUri.isEqual(node.currentUri)) {
                // 刷新前需要更新诊断信息数据
                this.decorationService.updateDiagnosisInfo(this.outlineTreeService.currentUri);
                // 因为Outline模块的节点是自展开的，不需要遍历
                await node.refresh();
                this.onDidRefreshedEmitter.fire();
            }
            (_c = this.refreshDeferred) === null || _c === void 0 ? void 0 : _c.resolve();
            this.refreshDeferred = null;
            this.onLoadingStateChangedEmitter.fire(false);
        });
    }
    dispose() {
        this.disposableCollection.dispose();
    }
};
OutlineModelService.DEFAULT_REFRESH_DELAY = 200;
OutlineModelService.DEFAULT_INIT_TREE_MODEL_DELAY = 500;
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], OutlineModelService.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.ILogger),
    tslib_1.__metadata("design:type", Object)
], OutlineModelService.prototype, "logger", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(outline_tree_service_1.OutlineTreeService),
    tslib_1.__metadata("design:type", outline_tree_service_1.OutlineTreeService)
], OutlineModelService.prototype, "outlineTreeService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.MarkerManager),
    tslib_1.__metadata("design:type", ide_core_browser_1.MarkerManager)
], OutlineModelService.prototype, "markerManager", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.IOutlineDecorationService),
    tslib_1.__metadata("design:type", Object)
], OutlineModelService.prototype, "decorationService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(outline_event_service_1.OutlineEventService),
    tslib_1.__metadata("design:type", outline_event_service_1.OutlineEventService)
], OutlineModelService.prototype, "outlineEventService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(browser_1.WorkbenchEditorService),
    tslib_1.__metadata("design:type", browser_1.WorkbenchEditorService)
], OutlineModelService.prototype, "editorService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.CommandService),
    tslib_1.__metadata("design:type", Object)
], OutlineModelService.prototype, "commandService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(document_symbol_1.DocumentSymbolStore),
    tslib_1.__metadata("design:type", document_symbol_1.DocumentSymbolStore)
], OutlineModelService.prototype, "documentSymbolStore", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_main_layout_1.IMainLayoutService),
    tslib_1.__metadata("design:type", Object)
], OutlineModelService.prototype, "mainLayoutService", void 0);
OutlineModelService = OutlineModelService_1 = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], OutlineModelService);
exports.OutlineModelService = OutlineModelService;
//# sourceMappingURL=outline-model.service.js.map